@@ -38,14 +38,17 @@
 ##
 ## Instead of returning a result for each argument, parcellfun returns
 ## only one cumulative result if "CumFunc" is non-empty. @var{cumfunc}
-## must be a function which performs an operation on two sets of outputs
-## of @var{fun} and returnes as many outputs as @var{fun}. If @var{nout}
-## is the number of outputs of @var{fun}, @var{cumfunc} gets a previous
-## output set of @var{fun} or of @var{cumfunc} as first @var{nout}
-## arguments and the current output of @var{fun} as last @var{nout}
-## arguments. The performed operation must be commutative and
-## associative. If the operation is e.g. addition, the result will be
-## the sum(s) of the outputs of @var{fun} over all calls of @var{fun}.
+## must be a function which performs an operation on two sets of
+## outputs of @var{fun} and returnes as many outputs as @var{fun}. If
+## @var{nout} is the number of outputs of @var{fun}, @var{cumfunc}
+## gets a previous output set of @var{fun} or of @var{cumfunc} as
+## first @var{nout} arguments and the current output of @var{fun} as
+## last @var{nout} arguments. The performed operation must be
+## mathematically commutative and associative. If the operation is
+## e.g. addition, the result will be the sum(s) of the outputs of
+## @var{fun} over all calls of @var{fun}. Since floating point
+## addition and multiplication are only approximately associative, the
+## cumulative result will not be exactly reproducible.
 ##
 ## Notice that jobs are served from a single first-come first-served queue,
 ## so the number of jobs executed by each process is generally unpredictable.
@@ -237,7 +240,7 @@ function varargout = parcellfun (nproc, fun, varargin)
     unwind_protect
       try
         ## re-seed random number states, adjusted for each process
-        seed *= iproc*bitmax;
+        seed *= iproc * (flintmax () - 1); # backwards compatibility
         cellfun (@ (fun) fun ("state", seed), random_func_list);
 
         ## child process. indicate ready state.
@@ -362,7 +365,7 @@ function varargout = parcellfun (nproc, fun, varargin)
         else
           isubp = -isubp;
           if (ready(isubp))
-            ## premature exit means an unhandled error occured in a subprocess.
+            ## premature exit means an unhandled error occurred in a subprocess.
             ## the process should have griped, we just try to exit gracefully.
             pending(isubp) = 0;
             terminated(isubp) = true;
