@@ -22,7 +22,7 @@ DEFUN_DLD (__exit__, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Loadable Function} __exit__ (status)\n\
 This is a wrapper over the POSIX _exit() system call. Calling this function\n\
-will terminate the running process immediatelly, bypassing normal Octave\n\
+will terminate the running process immediately, bypassing normal Octave\n\
 terminating sequence. It is suitable to terminate a forked process. It\n\
 should be considered expert-only and not to be used in normal code.\n\
 @end deftypefn") 
