@@ -1,7 +1,6 @@
 /*
 
-Copyright (C) 2002 Hayato Fujiwara <h_fujiwara@users.sourceforge.net>
-Copyright (C) 2010-2015 Olaf Till <i7tiol@t-online.de>
+Copyright (C) 2010-2016 Olaf Till <i7tiol@t-online.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -22,17 +21,9 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 // PKG_DEL: autoload ("network_get_info", "parallel_interface.oct", "remove");
 
 #include <octave/oct.h>
-
-#include "defun-dld.h"
-#include "dirfns.h"
-#include "error.h"
-#include "help.h"
-#include "oct-map.h"
-// #include "systime.h"
-#include "ov.h"
-#include "oct-obj.h"
-#include "utils.h"
-#include "oct-env.h"
+#include <octave/dirfns.h>
+#include <octave/oct-map.h>
+#include <octave/oct-env.h>
 
 #include "parallel-gnutls.h"
 
@@ -117,7 +108,7 @@ client, servers starting with @code{1}.\n\
 
   octave_scalar_map node_info (fields);
 
-  for (int i = 0; i < nconns; i++)
+  for (octave_idx_type i = 0; i < nconns; i++)
     {
       if (conns->get_connections ()(i)->is_pseudo_connection ())
         node_info.setfield ("local_machine", true);
