@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2015 Olaf Till <i7tiol@t-online.de>
+Copyright (C) 2015, 2016 Olaf Till <i7tiol@t-online.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -40,6 +40,16 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 
 #include <poll.h>
 
+
+enum encryption_type
+  {
+    ENCR_RAW,           // bare sockets
+    ENCR_TLS_SRP        // gnutls TLS SRP
+  };
+
+#define PROTOCOL_VERSION 1
+
+
 /*
 Although we use the standard file format, we can't rely on
 gnutls_srp_set_server_credentials_file(), since current gnutls uses
@@ -271,6 +281,183 @@ public:
       }
   }
 
+  int network_send_4byteint (uint32_t v, bool)
+  {
+    uint32_t t = htonl (v);
+
+    write ((void *) &t, 4);
+
+    if (good ())
+      return 0;
+    else
+      return -1;
+  }
+
+  int network_recv_4byteint (uint32_t &v)
+  {
+    uint32_t t;
+
+    read ((void *) &t, 4);
+
+    if (! good ())
+      return -1;
+
+    v = ntohl (t);
+
+    return 0;
+  }
+
+  int network_send_4byteint (int32_t v)
+  {
+    if (signed_int_rep () && v < 0)
+      {
+        once_gripe_neg_int_rep ();
+
+        return -1;
+      }
+
+    union
+    {
+      int32_t s;
+      uint32_t u;
+    } cast;
+
+    cast.s = v;
+
+    return network_send_4byteint (cast.u, true);
+  }
+
+  int network_recv_4byteint (int32_t &v)
+  {
+    uint32_t t;
+
+    union
+    {
+      int32_t s;
+      uint32_t u;
+    } cast;
+
+
+    read ((void *) &t, 4);
+
+    if (! good ())
+      return -1;
+
+    cast.u = ntohl (t);
+
+    v = cast.s;
+
+    if (signed_int_rep () && v < 0)
+      {
+        once_gripe_neg_int_rep ();
+
+        return -1;
+      }
+
+    return 0;
+  }
+
+  int network_send_string (const char *s)
+  {
+    uint32_t l = strlen (s);
+
+    uint32_t t = htonl (l);
+
+    write ((void *) &t, 4);
+
+    write ((const void *) s, l);
+
+    if (good ())
+      return 0;
+    else
+      return -1;
+  }
+
+  int network_recv_string (char *s, int maxl)
+  {
+    uint32_t l, t;
+
+    read ((void *) &t, 4);
+
+    if (! good ())
+      return -1;
+
+    l = ntohl (t);
+
+    if (l > maxl - 1)
+      {
+        c_verror ("string to be received would be longer than buffer");
+
+        return -1;
+      }
+
+    s[l] = '\0';
+
+    read ((void *) s, l);
+
+    if (good ())
+      return 0;
+    else
+      return -1;
+  }
+
+  int network_recv_string (std::string &str)
+  {
+    uint32_t l, t;
+
+    read ((void *) &t, 4);
+
+    if (! good ())
+      return -1;
+
+    l = ntohl (t);
+
+    struct __bufguard
+    {
+      __bufguard (octave_idx_type __l) : __buf (new char[__l]) {}
+      ~__bufguard (void) { delete [] __buf; }
+      char* __get (void) { return __buf; }
+      char *__buf;
+    } __bufg (l);
+
+    read (__bufg.__get (), l);
+
+    std::string tstr (__bufg.__get (), l);
+
+    str = tstr;
+
+    if (good ())
+      return 0;
+    else
+      return -1;
+  }    
+
+  static int signed_int_rep (void)
+  {
+    // I know this is paranoid, and that current Octave itself just
+    // relies on twos complement in saving/loading signed integers.
+
+    static int ret = -1;
+
+    if (ret == -1)
+      {
+        union
+        {
+          uint32_t u;
+          int32_t s;
+        } t;
+
+        t.s = -1;
+
+        if (t.u != 0xFFFFFFFF)
+          ret = 1;
+        else
+          ret = 0;
+      }
+
+    return ret;
+  }
+
 protected:
 
   int fid;
@@ -334,6 +521,17 @@ private:
   // destruction.
   char inbuf;
 
+  void once_gripe_neg_int_rep (void)
+  {
+    static bool first_time = true;
+
+    if (first_time)
+      {
+        c_verror ("This machine doesn't seem to use twos complement as negative integer representation. If you want this to be supported, please file a bug report.");
+
+        first_time = false;
+      }
+  }
 }; // class octave_parallel_streambuf
 
 // To avoid user space buffers, provided by external libraries, which
@@ -404,27 +602,26 @@ protected:
   BEGIN_INTERRUPT_IMMEDIATELY_IN_FOREIGN_CODE_2
 
 // TCP sockets without authentication or encryption, as an alternative
-// to a TLS connection, which is derived from this.
+// to encrypted connections (currently only TLS-SRP), which are (is)
+// derived from this.
 class
 octave_parallel_socket_streambuf : public octave_parallel_streambuf
 {
 public:
 
-  octave_parallel_socket_streambuf (int sid)
-  : undefined_connection_state (false)
+  // this needs c++11
+  octave_parallel_socket_streambuf (int sid, bool server)
+    : octave_parallel_socket_streambuf (sid, server, ENCR_RAW)
   {
-    fid = sid;
 
-    if (fid == -1)
-      {
-        store_errno = -1;
-
-        error ("no valid socket");
-      }
-    else
-      readable = writeable = true;
   }
 
+  // This is extra, so that (derived) encrypted connection streambufs
+  // can give it their own 'encrtype' argument to communicate and
+  // check encryption type before encryption starts.
+  octave_parallel_socket_streambuf (int sid, bool server,
+                                    encryption_type encrtype);
+
   ~octave_parallel_socket_streambuf (void)
   {
     if (fid >= 0)
@@ -644,68 +841,7 @@ octave_parallel_gnutls_streambuf : public octave_parallel_socket_streambuf
 public:
 
   octave_parallel_gnutls_streambuf
-  (int sid, octave_parallel_gnutls_srp_credentials *accred, bool server)
-    : octave_parallel_socket_streambuf (sid), ccred (accred), session (NULL)
-  {
-    // check result of parent class constructor, reset if necessary
-    if (! good ())
-      return;
-    else
-      readable = writeable = false;
-
-    dlprintf ("gnutls_steambuf ctor requests incr cred refcount\n");
-    if (! (cred = ccred->get_ref ()))
-      {
-        store_errno = -1;
-
-        error ("no valid credentials");
-
-        return;
-      }
-
-    if (gnutls_init (&session, server ? GNUTLS_SERVER : GNUTLS_CLIENT))
-      {
-        store_errno = -1;
-
-        error ("could not initialize gnutls session");
-
-        return;
-      }
-
-    if (gnutls_priority_set_direct
-        (session, "NORMAL:+SRP:+SRP-DSS:+SRP-RSA", NULL))
-      {
-        store_errno = -1;
-
-        error ("error in setting gnutls priorities");
-
-        return;
-      }
-
-    if (gnutls_credentials_set (session, GNUTLS_CRD_SRP, cred))
-      {
-        store_errno = -1;
-
-        error ("could not set credentials");
-
-        return;
-      }
-
-    gnutls_transport_set_ptr (session, (gnutls_transport_ptr_t) fid);
-
-    int ret;
-    ret = gnutls_handshake (session); // don't repeat interrupted handshake
-    if (ret)
-      {
-        store_errno = -1;
-
-        error ("handshake failed");
-
-        return;
-      }
-
-    readable = writeable = true;
-  }
+  (int sid, octave_parallel_gnutls_srp_credentials *accred, bool server);
 
   ~octave_parallel_gnutls_streambuf (void)
   {
@@ -874,143 +1010,37 @@ public:
 
   int network_send_4byteint (uint32_t v, bool) const
   {
-    uint32_t t = htonl (v);
-
-    strbuf->write ((void *) &t, 4);
-
-    if (strbuf->good ())
-      return 0;
-    else
-      return -1;
+    return strbuf->network_send_4byteint (v, true);
   }
 
   int network_recv_4byteint (uint32_t &v) const
   {
-    uint32_t t;
-
-    strbuf->read ((void *) &t, 4);
-
-    if (! strbuf->good ())
-      return -1;
-
-    v = ntohl (t);
-
-    return 0;
+    return strbuf->network_recv_4byteint (v);
   }
 
-  int network_send_4byteint (int32_t v)
+  int network_send_4byteint (int32_t v) const
   {
-    if (signed_int_rep () && v < 0)
-      {
-        once_gripe_neg_int_rep ();
-
-        return -1;
-      }
-
-    union
-    {
-      int32_t s;
-      uint32_t u;
-    } cast;
-
-    cast.s = v;
-
-    return network_send_4byteint (cast.u, true);
+    return strbuf->network_send_4byteint (v);
   }
 
-  int network_recv_4byteint (int32_t &v)
+  int network_recv_4byteint (int32_t &v) const
   {
-    uint32_t t;
-
-    union
-    {
-      int32_t s;
-      uint32_t u;
-    } cast;
-
-
-    strbuf->read ((void *) &t, 4);
-
-    if (! strbuf->good ())
-      return -1;
-
-    cast.u = ntohl (t);
-
-    v = cast.s;
-
-    if (signed_int_rep () && v < 0)
-      {
-        once_gripe_neg_int_rep ();
-
-        return -1;
-      }
-
-    return 0;
+    return strbuf->network_recv_4byteint (v);
   }
 
   int network_send_string (const char *s) const
   {
-    uint32_t l = strlen (s);
-
-    uint32_t t = htonl (l);
-
-    strbuf->write ((void *) &t, 4);
-
-    strbuf->write ((const void *) s, l);
-
-    if (strbuf->good ())
-      return 0;
-    else
-      return -1;
+    return strbuf->network_send_string (s);
   }
 
   int network_recv_string (char *s, int maxl) const
   {
-    uint32_t l, t;
-
-    strbuf->read ((void *) &t, 4);
-
-    if (! strbuf->good ())
-      return -1;
-
-    l = ntohl (t);
-
-    if (l > maxl - 1)
-      {
-        error ("string to be received would be longer than buffer");
-
-        return -1;
-      }
-
-    s[l] = '\0';
-
-    strbuf->read ((void *) s, l);
-
-    if (strbuf->good ())
-      return 0;
-    else
-      return -1;
+    return strbuf->network_recv_string (s, maxl);
   }
 
   int network_recv_string (std::string &str) const
   {
-    uint32_t l, t;
-
-    strbuf->read ((void *) &t, 4);
-
-    if (! strbuf->good ())
-      return -1;
-
-    l = ntohl (t);
-
-    str.resize (l);
-
-    strbuf->read ((void *) str.c_str (), l);
-
-    if (strbuf->good ())
-      return 0;
-    else
-      return -1;
+    return strbuf->network_recv_string (str);
   }    
 
   octave_parallel_streambuf *get_strbuf (void)
@@ -1020,28 +1050,7 @@ public:
 
   static int signed_int_rep (void)
   {
-    // I know this is paranoid, and that current Octave itself just
-    // relies on twos complement in saving/loading signed integers.
-
-    static int ret = -1;
-
-    if (ret == -1)
-      {
-        union
-        {
-          uint32_t u;
-          int32_t s;
-        } t;
-
-        t.s = -1;
-
-        if (t.u != 0xFFFFFFFF)
-          ret = 1;
-        else
-          ret = 0;
-      }
-
-    return ret;
+    return octave_parallel_streambuf::signed_int_rep ();
   }
 
 private:
@@ -1051,19 +1060,6 @@ private:
   std::istream *istr;
 
   std::ostream *ostr;
-
-  void once_gripe_neg_int_rep (void)
-  {
-    static bool first_time = true;
-
-    if (first_time)
-      {
-        error ("This machine doesn't seem to use twos complement as negative integer representation. If you want this to be supported, please file a bug report.");
-
-        first_time = false;
-      }
-  }
-
 };
 
 #endif // __OCT_PARALLEL_STREAMS__
