@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2015 Olaf Till <i7tiol@t-online.de>
+Copyright (C) 2015, 2016 Olaf Till <i7tiol@t-online.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -96,10 +96,6 @@ extern unsigned long int num_processors (enum nproc_query query);
 
 #include <octave/ov-struct.h>
 
-#include <octave/load-save.h>
-
-#include <octave/ls-oct-binary.h>
-
 
 #define N_CONNECT_RETRIES 10
 
@@ -133,6 +129,8 @@ extern unsigned long int num_processors (enum nproc_query query);
 
 // parallel package includes
 
+#include "error-helpers.h"
+
 #ifdef HAVE_LIBGNUTLS
 #include "sensitive-data.h"
 #endif
@@ -143,6 +141,7 @@ extern unsigned long int num_processors (enum nproc_query query);
 
 #include "p-sighandler.h"
 
+#include "minimal-load-save.h"
 
 #ifdef HAVE_LIBGNUTLS
 void parallel_gnutls_set_mem_functions (void);
