@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2015 Olaf Till <i7tiol@t-online.de>
+Copyright (C) 2015, 2016 Olaf Till <i7tiol@t-online.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -81,15 +81,11 @@ shutdown.\n\
     }
 
   // get username
-  std::string username (args(0).string_value ());
+  std::string username;
 
-  if (error_state)
-    {
-      error ("%s: first argument can not be converted to a string",
-             fname.c_str ());
-
-      return retval;
-    }
+  CHECK_ERROR (username = args(0).string_value (), retval,
+               "%s: first argument can not be converted to a string",
+               fname.c_str ());
 
   // default options
   bool clientfile (true);
@@ -99,15 +95,9 @@ shutdown.\n\
     {
       octave_scalar_map options;
 
-      options = args(1).scalar_map_value ();
-
-      if (error_state)
-        {
-          error ("%s: could not convert second argument to scalar structure",
-                 fname.c_str ());
-
-          return retval;
-        }
+      CHECK_ERROR (options = args(1).scalar_map_value (), retval,
+                   "%s: could not convert second argument to scalar structure",
+                   fname.c_str ());
 
       octave_value tmp;
 
@@ -116,15 +106,9 @@ shutdown.\n\
 
       if (tmp.is_defined ())
         {
-          clientfile = tmp.bool_value ();
-
-          if (error_state)
-            {
-              error ("%s: could not convert option 'unattended' to bool",
-                     fname.c_str ());
-
-              return retval;
-            }
+          CHECK_ERROR (clientfile = tmp.bool_value (), retval,
+                       "%s: could not convert option 'unattended' to bool",
+                       fname.c_str ());
         }
     } // args.length () == 2
 
@@ -135,24 +119,14 @@ shutdown.\n\
 
   octave_value_list f_ret;
 
-  f_ret = feval ("pkg", f_args, 1);
+  CHECK_ERROR (f_ret = feval ("pkg", f_args, 1), retval,
+               "%s: could not get prefix from pkg", fname.c_str ());
 
-  if (error_state)
-    {
-      error ("%s: could not get prefix from pkg", fname.c_str ());
+  std::string basedir;
 
-      return retval;
-    }
-
-  std::string basedir = f_ret(0).string_value ();
-
-  if (error_state)
-    {
-      error ("%s: could not convert output of pkg ('prefix') to string)",
-             fname.c_str ());
-
-      return retval;
-    }
+  CHECK_ERROR (basedir = f_ret(0).string_value (), retval,
+               "%s: could not convert output of pkg ('prefix') to string)",
+               fname.c_str ());
 
   basedir = basedir + file_ops::dir_sep_str () +
     "parallel-srp-data" + file_ops::dir_sep_str ();
