@@ -1,7 +1,7 @@
 /*
 
 Copyright (C) 2002 Hayato Fujiwara <h_fujiwara@users.sourceforge.net>
-Copyright (C) 2010-2015 Olaf Till <i7tiol@t-online.de.de>
+Copyright (C) 2010-2016 Olaf Till <i7tiol@t-online.de.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 #include "config.h"
 
 #include <octave/oct.h>
-#include <oct-env.h>
+#include <octave/oct-env.h>
 
 #include <sys/socket.h>
 #include <errno.h>
@@ -55,7 +55,9 @@ DEFUN_DLD (pconnect, args, nargout,
 Connects to a network of parallel cluster servers.\n\
 \n\
 As a precondition, a server must have been started at each machine of\n\
-the cluster, see @code{pserver}.\n\
+the cluster, see @code{pserver}. Connections are not guaranteed to\n\
+work if client and server are from @code{parallel} packages of\n\
+different versions, so versions should be kept equal.\n\
 \n\
 @var{hosts} is a cell-array of strings, holding the names of all\n\
 server machines. The machines must be unique, and their names must be\n\
@@ -132,7 +134,7 @@ sclose (sockets);\n\
 @end group\n\
 @end example\n\
 \n\
-@seealso{pserver, reval, psend, precv, sclose, parallel_generate_srp_data, select_sockets, rfefal}\n\
+@seealso{pserver, reval, psend, precv, sclose, parallel_generate_srp_data, select_sockets, rfeval}\n\
 @end deftypefn")
 {
   std::string fname ("pconnect");
@@ -175,17 +177,15 @@ sclose (sockets);\n\
   // The new cell-array argument does not give the client name; so it
   // has one entry less than the previous character array would have
   // had rows.
-  Array<std::string> hosts = args(0).cellstr_value ();
-  if (error_state)
+  Array<std::string> hosts;
+  bool err;
+  SET_ERR (hosts = args(0).cellstr_value (), err);
+  if (err)
     {
-      error_state = 0;
-      charMatrix cm = args(0).char_matrix_value ();
-      if (error_state)
-        {
-          error ("%s: first argument must be a cell array of strings or a character matrix",
-                 fname.c_str ());
-          return retval;
-        }
+      charMatrix cm;
+      CHECK_ERROR (cm = args(0).char_matrix_value (), retval,
+                   "%s: first argument must be a cell array of strings or a character matrix",
+                   fname.c_str ());
       int rows = cm.rows ();
       int cols = cm.columns ();
       cm = cm.transpose (); 
@@ -251,15 +251,9 @@ sclose (sockets);\n\
     {
       octave_scalar_map options;
 
-      options = args(1).scalar_map_value ();
-
-      if (error_state)
-        {
-          error ("%s: could not convert second argument to scalar structure",
-                 fname.c_str ());
-
-          return retval;
-        }
+      CHECK_ERROR (options = args(1).scalar_map_value (), retval,
+                   "%s: could not convert second argument to scalar structure",
+                   fname.c_str ());
 
       octave_value tmp;
 
@@ -268,15 +262,9 @@ sclose (sockets);\n\
 
       if (tmp.is_defined ())
         {
-          use_gnutls = tmp.bool_value ();
-
-          if (error_state)
-            {
-              error ("%s: could not convert option 'use_tls' to bool",
-                     fname.c_str ());
-
-              return retval;
-            }
+          CHECK_ERROR (use_gnutls = tmp.bool_value (), retval,
+                       "%s: could not convert option 'use_tls' to bool",
+                       fname.c_str ());
         }
 
       // custom password file
@@ -284,15 +272,9 @@ sclose (sockets);\n\
 
       if (tmp.is_defined ())
         {
-          cpfile = tmp.string_value ();
-
-          if (error_state)
-            {
-              error ("%s: could not convert option 'password_file' to string",
-                     fname.c_str ());
-
-              return retval;
-            }
+          CHECK_ERROR (cpfile = tmp.string_value (), retval,
+                       "%s: could not convert option 'password_file' to string",
+                       fname.c_str ());
         }
 
       // user name
@@ -300,15 +282,9 @@ sclose (sockets);\n\
 
       if (tmp.is_defined ())
         {
-          user = tmp.string_value ();
-
-          if (error_state)
-            {
-              error ("%s: could not convert option 'user' to string",
-                     fname.c_str ());
-
-              return retval;
-            }
+          CHECK_ERROR (user = tmp.string_value (), retval,
+                       "%s: could not convert option 'user' to string",
+                       fname.c_str ());
 
           if (user.length ())
             use_pfile = false;
@@ -381,20 +357,13 @@ sclose (sockets);\n\
                 {
                   octave_value_list f_args (1);
                   f_args(0) = octave_value ("prefix");
-                  octave_value_list f_ret = feval ("pkg", f_args, 1);
-                  if (error_state)
-                    {
-                      error ("%s: could not get prefix from pkg",
-                             fname.c_str ());
-                      return retval;
-                    }
-                  cpfile = f_ret(0).string_value ();
-                  if (error_state)
-                    {
-                      error ("%s: could not convert output of pkg ('prefix') to string)",
-                             fname.c_str ());
-                      return retval;
-                    }
+                  octave_value_list f_ret;
+                  CHECK_ERROR (f_ret = feval ("pkg", f_args, 1), retval,
+                               "%s: could not get prefix from pkg",
+                               fname.c_str ());
+                  CHECK_ERROR (cpfile = f_ret(0).string_value (), retval,
+                               "%s: could not convert output of pkg ('prefix') to string)",
+                               fname.c_str ());
                   cpfile = cpfile + file_ops::dir_sep_str () +
                     "parallel-srp-data" + file_ops::dir_sep_str () +
                     "client" + file_ops::dir_sep_str () + "user_passwd";
@@ -494,7 +463,7 @@ sclose (sockets);\n\
             break;
           else if (errno != ECONNREFUSED && errno != EINTR)
             {
-              error ("connect error");
+              c_verror ("connect error");
               break;
             }
           else
@@ -526,7 +495,7 @@ sclose (sockets);\n\
 #endif
             conn->insert_cmd_stream
               (new octave_parallel_stream
-               (new octave_parallel_socket_streambuf (sock)));
+               (new octave_parallel_socket_streambuf (sock, false)));
           __sockg.__release ();
           if (! conn->get_cmd_stream ()->good ())
             {
@@ -641,7 +610,7 @@ sclose (sockets);\n\
 #endif
                 conns->get_connections ()(i + 1)->insert_data_stream
                   (data_stream = new octave_parallel_stream
-                   (new octave_parallel_socket_streambuf (sock)));
+                   (new octave_parallel_socket_streambuf (sock, false)));
               __sockg.__release ();
               if (! data_stream->good ())
                 {
@@ -686,9 +655,10 @@ sclose (sockets);\n\
                 }
               else
                 {
-                  write_header (data_stream->get_ostream (), LS_BINARY);
-                  conns->get_connections ()(i + 1)->connection_read_header ();
-                  if (error_state || ! data_stream->good ())
+                  minimal_write_header (data_stream->get_ostream ());
+                  if (conns->get_connections ()(i + 1)->
+                      connection_read_header () ||
+                      ! data_stream->good ())
                     {
                       error ("communication error in initialization");
                       return retval;
@@ -700,7 +670,7 @@ sclose (sockets);\n\
             }
           else if (errno != ECONNREFUSED && errno != EINTR)
             {
-              error ("connect error");
+              c_verror ("connect error");
               break;
             }
           else
