@@ -21,6 +21,8 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 // PKG_ADD: autoload ("pserver", "parallel_interface.oct");
 // PKG_DEL: autoload ("pserver", "parallel_interface.oct", "remove");
 
+#include "config.h"
+
 #include <octave/oct.h>
 #include <octave/symtab.h>
 #include <octave/oct-env.h>
@@ -28,7 +30,14 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 #include <octave/sighandlers.h>
 #include <octave/parse.h>
 #include <octave/cmd-edit.h>
+#ifdef HAVE_OCTAVE_INTERPRETER_H
+// >= octave-4.2
+#include <octave/octave.h>
+#include <octave/interpreter.h>
+#else
+// <= octave-4.0..
 #include <octave/toplev.h>
+#endif
 
 #include <sys/socket.h>
 #include <sys/types.h>
@@ -47,8 +56,6 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 
 #include "parallel-gnutls.h"
 
-#include "config.h"
-
 // Octave > 3.2.4 does not have these in a header file, but in
 // sighandlers.cc, and uses gnulib:: for these. So this is copied from
 // Octave-3.2.4.
@@ -1101,7 +1108,11 @@ the swap device even after shutdown of the application.\n\
 
           __pwg.__free ();
 
+#ifdef HAVE_OCTAVE_INTERPRETER_H
+          octave::application::the_interpreter () -> interactive (false);
+#else
           interactive = false;
+#endif
 
           // install 'sockets' as Octave variable
           symbol_table::assign ("sockets", sockets);
