@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2015 Olaf Till <i7tiol@t-online.de>
+Copyright (C) 2015, 2016 Olaf Till <i7tiol@t-online.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,15 +21,9 @@ along with this program; If not, see <http://www.gnu.org/licenses/>.
 // PKG_DEL: autoload ("select_sockets", "parallel_interface.oct", "remove");
 
 #include <octave/oct.h>
-#include "defun-dld.h"
-#include "dirfns.h"
-#include "error.h"
-#include "help.h"
-#include "oct-map.h"
-#include "ov.h"
-#include "oct-obj.h"
-#include "utils.h"
-#include "oct-env.h"
+#include <octave/dirfns.h>
+#include <octave/oct-map.h>
+#include <octave/oct-env.h>
 
 #include "parallel-gnutls.h"
 
@@ -87,9 +81,12 @@ received.\n\
 
   double atimeout = args(1).double_value ();
   int nfds = FD_SETSIZE;
+  bool err;
   if (args.length () == 3)
-    nfds = args(2).int_value ();
-  if (error_state)
+    {
+      SET_ERR (nfds = args(2).int_value (), err);
+    }
+  if (err)
     {
       print_usage ();
 
@@ -220,15 +217,15 @@ received.\n\
               int err;
               if ((err = conns->get_connections ()(i)->poll_for_errors ()) < 0)
                 {
+                  conns->close ();
                   error ("%s: communication error with server with index %i",
                          fname.c_str (), i + 1);
-                  conns->close ();
                   return retval;
                 }
               else if (err > 0)
                 {
-                  error ("%s: a previous command at server with index %i caused an error",
-                         fname.c_str (), i + 1);
+                  c_verror ("%s: a previous command at server with index %i caused an error",
+                            fname.c_str (), i + 1);
                   command_errors = true;
                 }
             }
@@ -236,7 +233,11 @@ received.\n\
     }
 
   if (command_errors)
-    return retval;
+    {
+      error ("command error");
+      
+      return retval;
+    }
 
   return octave_value (ridx);
 }
